@@ -43,6 +43,7 @@
 #include "command.h"
 #include "splash-widget.h"
 #include "util.h"
+#include "gsm-dbus.h"
 #include "gsm-sound.h"
 #include "gsm-gsd.h"
 #include "gsm-proxy.h"
@@ -336,6 +337,7 @@ main (int argc, char *argv[])
   char **versions;
   GConfClient *gconf_client;
   GOptionContext *goption_context;
+  gboolean dbus_daemon_owner;
   
   if (getenv ("GSM_VERBOSE_DEBUG"))
     gsm_set_verbose (TRUE);
@@ -437,6 +439,7 @@ main (int argc, char *argv[])
 
   /* Need DISPLAY set */
   gsm_keyring_daemon_start ();
+  dbus_daemon_owner = gsm_dbus_daemon_start ();
   gsm_gsd_start ();
   
   /* Read the rest of config options */  
@@ -498,9 +501,6 @@ main (int argc, char *argv[])
 
   start_session (the_session);
 
-  if (a_t_support) /* the ATs are happier if the session has started */
-    gsm_assistive_technologies_start ();
-
   gtk_main ();
 
   gsm_remote_desktop_cleanup ();
@@ -509,6 +509,9 @@ main (int argc, char *argv[])
 
   gsm_keyring_daemon_stop ();
 
+  if (dbus_daemon_owner)
+    gsm_dbus_daemon_stop ();
+
   g_object_unref (gconf_client);
   gsm_shutdown_gconfd ();
 
