@@ -38,15 +38,19 @@ static gboolean zap = FALSE;
 /* True if we should use dialog boxes */
 static gboolean gui = FALSE; 
 
+/* True if we should do the requested action without confirmation */
+static gboolean silent = FALSE; 
+
 static char *session_name = NULL;
 
 static IceConn ice_conn = NULL;
 
-static const struct poptOption options[] = {
-  {"session-name", 's', POPT_ARG_STRING, &session_name, 0, N_("Set the current session"), NULL},
-  {"kill", '\0', POPT_ARG_NONE, &zap, 0, N_("Kill session"),     NULL},
-  {"gui",  '\0', POPT_ARG_NONE, &gui, 0, N_("Use dialog boxes"), NULL},
-  {NULL,   '\0', 0, NULL, 0}
+static const GOptionEntry options[] = {
+  {"session-name", 's', 0, G_OPTION_ARG_STRING, &session_name, N_("Set the current session name"), N_("NAME")},
+  {"kill", '\0', 0, G_OPTION_ARG_NONE, &zap, N_("Kill session"), NULL},
+  {"gui",  '\0', 0, G_OPTION_ARG_NONE, &gui, N_("Use dialog boxes for errors"), NULL},
+  {"silent", '\0', 0, G_OPTION_ARG_NONE, &silent, N_("Do not require confirmation"), NULL},
+  {NULL}
 };
 
 static int exit_status = 0;
@@ -76,8 +80,8 @@ save_complete (GnomeClient* client, gpointer data)
      to.  Some of them aren't particularly useful.  Interestingly,
      there is no way to request a shutdown without a save.  */
   gnome_client_request_save (client, GNOME_SAVE_BOTH, zap,
-			     GNOME_INTERACT_ANY, 0, 1);
-  
+			     silent ? GNOME_INTERACT_NONE : GNOME_INTERACT_ANY,
+			     0, 1);
   
   ice_ping ();
 }
@@ -98,7 +102,7 @@ cancelled_cb (GnomeClient *client, gpointer data)
 static void
 display_error (const char *message)
 {
-  if (gui) 
+  if (gui && !silent) 
     {
       GtkWidget *dialog;
       
@@ -156,6 +160,7 @@ int
 main (int argc, char *argv[])
 {
   GnomeClient *client;
+  GOptionContext *goption_context;
 
   /* Initialize the i18n stuff */
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -165,9 +170,12 @@ main (int argc, char *argv[])
 
   IceAddConnectionWatch (ice_connection_watch, NULL);
 
+  goption_context = g_option_context_new (_("- Save the current session"));
+  g_option_context_add_main_entries (goption_context, options, GETTEXT_PACKAGE);
+
   gnome_program_init ("gnome-session-save", VERSION, LIBGNOMEUI_MODULE,
 		      argc, argv,
-		      GNOME_PARAM_POPT_TABLE, options,
+		      GNOME_PARAM_GOPTION_CONTEXT, goption_context,
 		      GNOME_PROGRAM_STANDARD_PROPERTIES,
 		      NULL);
 
